/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.spark.sql.hive.client

import scala.collection.immutable.IndexedSeq

import org.apache.commons.lang3.{JavaVersion, SystemUtils}

private[client] trait HiveClientVersions {
  private val testVersions = sys.env.get("SPARK_TEST_HIVE_CLIENT_VERSIONS")
  protected val versions = if (testVersions.nonEmpty) {
    testVersions.get.split(",").map(_.trim).filter(_.nonEmpty).toIndexedSeq
  } else if (SystemUtils.isJavaVersionAtLeast(JavaVersion.JAVA_9)) {
    IndexedSeq("2.0", "2.1", "2.2", "2.3", "3.0", "3.1")
  } else {
    IndexedSeq("0.12", "0.13", "0.14", "1.0", "1.1", "1.2", "2.0", "2.1", "2.2", "2.3", "3.0",
      "3.1")
  }
}
